def validate_webapp_data(webapp_data: dict[str, str], secret_key: bytes) -> bool:
    """
    Check if the bot is authorized to send messages to the user

    :param bot_token: Telegram bot token
    :param webapp_data: Webapp data

    :return: True if the bot is authorized to send messages to the user, False otherwise
    """


def get_secret_key(bot_token: str) -> bytes:
    """
    Get the secret key
    
    :param bot_token: Telegram bot token

    :return: The secret key
    """
    
def parse_webapp_data(webapp_data: str) -> dict[str, str]:
    """
    Parse the webapp data
    
    :param webapp_data: Webapp data

    :return: The parsed webapp data
    """